@@ -4,7 +4,7 @@ from json import dumps, JSONEncoder
 
 from .exceptions import RepositoryError
 from .utils import Fault
-from .utils.dicts import ATOMIC_TYPES, map_dict_keys, merge_dict, value_at_key_path
+from .utils.dicts import ATOMIC_TYPES, map_dict_keys, merge_dict, untoml, value_at_key_path
 from .utils.text import force_text, mark_for_translation as _
 
 
@@ -27,35 +27,25 @@ class DoNotRunAgain(Exception):
 
 def deepcopy_metadata(obj):
     """
-    Our own version of deepcopy.copy that doesn't pickle.
+    Our own version of deepcopy.copy that doesn't pickle since some
+    Fault callbacks are unpicklable.
     """
     if isinstance(obj, METADATA_TYPES):
         return obj
     elif isinstance(obj, dict):
-        if isinstance(obj, ATOMIC_TYPES[dict]):
-            new_obj = atomic({})
-        else:
-            new_obj = {}
-        for key, value in obj.items():
-            new_key = copy(key)
-            new_obj[new_key] = deepcopy_metadata(value)
-    elif isinstance(obj, (list, tuple)):
-        if isinstance(obj, (ATOMIC_TYPES[list], ATOMIC_TYPES[tuple])):
-            new_obj = atomic([])
-        else:
-            new_obj = []
-        for member in obj:
-            new_obj.append(deepcopy_metadata(member))
-    elif isinstance(obj, set):
-        if isinstance(obj, ATOMIC_TYPES[set]):
-            new_obj = atomic(set())
-        else:
-            new_obj = set()
-        for member in obj:
-            new_obj.add(deepcopy_metadata(member))
+        new_obj = {
+            copy(key): deepcopy_metadata(value) for key, value in obj.items()
+        }
+    elif isinstance(obj, (list, tuple, set)):
+        new_obj = [
+            deepcopy_metadata(value) for value in obj
+        ]
     else:
         assert False  # there should be no other types
-    return new_obj
+
+    # Try to preserve the original type, even if its a superclass of
+    # dict, list, tuple or set.
+    return type(obj)(new_obj)
 
 
 def validate_metadata(metadata, _top_level=True):
